#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>

#include "macros.h"
#include "f2c.h"
#include "clapack.h"

/* code for handling CLAPACK errors as reported in the INFO variable.
   the error reporting is identical for dgetrf and dgetri */
#define my_handle_errors(info, fcn)				  \
  if(info != 0) {						  \
    if(info < 0) { /* invalid argument */			  \
      DIE(#fcn "_() reports invalid arg %d\n", (int)-info);	  \
    } else {							  \
      DIE(#fcn "_() reports singular matrix with U(%d,%d) = 0\n", \
	  (int)info, (int)info);				  \
    }								  \
  }



/* Compute the log of the *absolute value* of the determinant.  NB:
   this will overwrite A with its LU decomposition */
double det(double* A, integer LDA, integer n) {
  int i;

  assert(A != NULL); assert(n > 0); assert(LDA >= n);

  /* static storage for P (row-permuatation) array */
  static integer* P = NULL;
  static int P_size = 0;
  if(P_size < n) {
    P = (integer*)realloc(P, (P_size = n)*sizeof(integer));
    assert(P != NULL);
  }

  integer INFO = 0;

  /* perform LU factorization (overwritten on A) */
  dgetrf_(&n, &n, A, &LDA, P, &INFO);
  my_handle_errors(INFO, dgetrf);

  /* compute determinant as product of diagonal of U (which is stored
     as the upper triangle of A) */
  double logdet = 0;
  const int i_max = n*LDA;
  const int i_inc = LDA + 1;
  for(i = 0; i < i_max; i += i_inc) {
    const double A_i = A[i];
    assert(isfinite(A_i));
    logdet += log(A_i < 0 ? -A_i : A_i);  /* absolute value */
  }

  return logdet;
}

#undef my_handle_errors /* clean up */
