#include <assert.h>

#include "linalg.h"

void make_laplacian(double* const restrict A, const int n) {
  assert(A != NULL); assert(n > 0);
  int i, j;
  /* matrix storage is column-major, so the outer iteration is over
     j instead of i */
  for(j = 0; j < n; ++j) {
    const int col_start = j*n;
    double sum = 0;
    for(i = 0; i < n; ++i) {
      const int idx = col_start + i;
      if(i == j) {
	assert(A[idx] == 0);  /* proper adj matrix has zero diagonal */
      } else {
	const double val = A[idx];
	assert(val >= 0);
	A[idx] = -val;        /* negate edges in L */
	sum += val;           /* accumulate in-weight */
      }
    }
    A[col_start + j] = sum;   /* diagonal L_jj = in-weight */
  }
}
