#!/usr/bin/perl -w

my $UTIL = $ENV{"HOME"}."/research/util";

my $RELATIVE = 0;
my $SIGNED = 0;

while($ARGV[0] =~ m/^-/o) {
    my $arg = shift(@ARGV);
    if($arg eq '-r') {
	$RELATIVE = 1;
    } elsif($arg eq '-s') {
	$SIGNED = 1;
    } else {
	die "malformed option \"$arg\"";
    }
}

if(scalar(@ARGV) != 2) {
    print STDERR "usage: ./numeric_diff.pl [-r] file1[.gz] file2[.gz]\n";
    exit 1;
}

my ($f1, $f2) = @ARGV;
my ($fname1, $fname2) = map {
    (m/\.gz$/io ? "gunzip -c" : "cat")." \"$_\" | $UTIL/hex2dec |";
} ($f1, $f2);
open(F1, $fname1) or die "couldn't open \"$f1\"";
open(F2, $fname2) or die "couldn't open \"$f2\"";

my $maxdiff = 0;
my $l1diff = 0;
my $l2diff = 0;
my $line_num = 0;
while(1) {
    ++$line_num;
    my $line1 = <F1>;
    my $line2 = <F2>;
    die "line $line_num: $f1 is longer than $f2" 
	if(defined($line1) && !defined($line2));
    die "line $line_num: $f2 is longer than $f1" 
	if(defined($line2) && !defined($line1));
    last if(!defined($line1) && !defined($line2));
    my @elts1 = split(' ', $line1);
    my @elts2 = split(' ', $line2);
    die "line $line_num: unequal elements: ".
	scalar(@elts1)." vs ".scalar(@elts2) 
	if(scalar(@elts1) != scalar(@elts2));
    my @diffed = map {
	my ($e1, $e2) = ($elts1[$_], $elts2[$_]);
	if($e1 !~ m/^[0-9e\.+-]+$/io || $e2 !~ m/^[0-9e\.+-]+$/io) {
	    die "line $line_num: mismatched elements: \"$e1\" vs \"$e2\""
		if($e1 ne $e2);
	    $e1;
	} else {
	    $diff = $e1 - $e2;
	    $diff = abs($diff) if(!$SIGNED);

	    # relative differences are not easy to define.  here's
	    # what i've come up with:
	    # sqrt[(x - y)^2 / 2*(x^2 + y^2)]
#	    $diff = sqrt(($diff*$diff)/(2*($e1*$e1 + $e2*$e2))) 
#		if($RELATIVE && $diff > 0);  # guard against div-by-0

	    # the above is inadequate for large or small numbers
	    # because squaring will cause overflow or underflow.
	    # therefore, we use a linear substitute
	    $diff = 2*$diff/(abs($e1) + abs($e2))
		if($RELATIVE && $diff != 0);  # guard against div-by-0
	    $maxdiff = $diff if($diff > $maxdiff);
	    $l1diff += abs($diff);
	    $l2diff += $diff*$diff;
	    $diff;
	}
    } (0 .. $#elts1);
    print "@diffed\n";
}
$l2diff = sqrt($l2diff);
print STDERR "maxdiff = $maxdiff  l1diff = $l1diff  l2diff = $l2diff\n";

close(F1); close(F2);
