#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "linalg.h"

int main(const int argc, const char* argv[]) {
  if(argc != 3) {
    fprintf(stderr, "usage: %s <matrix-file> <n>\n", argv[0]);
    exit(1);
  }
  FILE* mat = fopen(argv[1], "r");
  assert(mat != NULL);
  const int n = atoi(argv[2]);
  assert(n > 0);

  double* A = read_sized_matrix(mat, n, n);
  assert(fclose(mat) == 0);

  int i;
  const int end = n*n;
  double* B = (double*)malloc(n*n*sizeof(double));
  for(i = 0; i < end; ++i) { B[i] = A[i]; }

  long double d = expl(det_inv(A, n, n));
  long double d2 = expl(det(B, n, n));
  const long double relative_diff = 2.0*(d > d2 ? d - d2 : d2 - d)/(d + d2);
  assert(relative_diff < 1e-12);

  printf("%.15Lg\n", d);
  write_matrix_dec(stdout, A, n, n);

  free(A); free(B);

  return 0;
}
