#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "matrixtree.h"
#include "linalg.h"

int main(const int argc, const char* argv[]) {
  if(argc != 3) {
    fprintf(stderr, "usage: %s <adj-matrix-file> <n>\n",
	    argv[0]);
    exit(1);
  }

  FILE* mat = fopen(argv[1], "r");
  assert(mat != NULL);
  const int n = atoi(argv[2]);
  assert(n > 0);

  double* A = read_sized_matrix(mat, n, n);
  assert(fclose(mat) == 0);

  /* convert to log-space */
  int i;
  const int end = n*n;
  for(i = 0; i < end; ++i) { A[i] = log(A[i]); }

  double* B = (double*)malloc(n*n*sizeof(double));
  for(i = 0; i < end; ++i) { B[i] = A[i]; }

  long double par = expl(sf_par_mar(A, n));
  long double par2 = expl(sf_par(B, n));
  //  assert(par == par2);

  printf("%.15Lg\n", par);
  printf("%.15Lg\n", par2);
  write_matrix_dec(stdout, A, n, n);

  free(A); free(B);

  return 0;
}
